# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

#from builtins import object
class element(object):
    """
    Abstract class specifying the common framework for all individuals and forms.

    **Object Parameters:**
        **assoc** (:class:`~sortal.element.element`): another element (individual/form) this element belong to or accociate with.
        **sort** (:class:`~sortal.sort.sort.sort`): sort defining the element.
    """
    def __init__(self, sort):
        """
        initializing the element. This function is called for all individual and form initializations
        an element (individual or form) needs to belong to a sort
        an element can also be associated with another element: the element belongs
        to a form or the element is a codescriptor of another element

        :param sort: sort of the individual
        :type sort: ~sortal.sort.sort.sort

        """
        self.assoc = None
        self.sort = sort
        
    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        """
        check equality
        """
        if other is None or other.__class__ != self.__class__ or not other.sort == self.sort:
            return False
        return self.equals(other)

    def equals(self, other):
        """
        check equality (without checking class or sort)
        """
        raise NotImplementedError()
        
    def __ne__(self, other):
        """
        check inequality
        """
        return not self == other
    
    def __lt__(self, other):
        """
        less than
        """
        if self.compareValue(other) == -1:
            return True
        return False

    def __le__(self, other):
        """
        less than or equals to
        """
        if self.compareValue(other) < 1 :
            return True
        return False

    def __gt__(self, other):
        """
        greater than
        """
        if self.compareValue(other) == 1:
            return True
        return False

    def __ge__(self, other):
        """
        greater than or equals to
        """
        if self.compareValue(other) > -1:
            return True
        return False
        
    def accept(self):
        """
        Implemented in each individual/form
        """
        raise NotImplementedError 
        
    def compareValue(self):
        """
        Implemented in each individual/form
        """
        raise NotImplementedError 
                
    def edit(self, editor):
        """
        Implemented in each individual/form
        """
        raise NotImplementedError
    
    def transform(self, tfSet):
        """
        transforming the element using a transformation set, 
        essentially editing the element with the transform set as editor

        :param tfSet: the transformation set to be applied
        :type tfSet: ~sortal.transform.transformSet.transformSet
        :returns: the transformed object
        """
        return self.edit(tfSet)
        
    def duplicate(self):
        """
        duplicating the element
        """
        if 'ind' in self.type:
            return self.duplicateInd()
        else:
            return self.duplicateForm()
                