# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import str

from builtins import object

class fragment(object):
    """
    fragment class 
    
    Compilation of object under the same sort for rule analysis
    """

    def __init__(self, sort):
        self.sort = sort
        self.individuals = None
        self.distinguishables = None
        self.graphNodes = None
        if 'distinguishable' in self.mapping().type:
            self.distinguishables = []
        elif self.mapping().type == 'map-graphMap':
            self.graphNodes = []
        else: 
            self.individuals = []
        self.visited = False


    def __str__(self):
        if self.individuals is not None:
            string = 'inds'
            for i in self.individuals:
                string+= '|'+str(i)
            return string
        if self.graphNodes is not None:
            string = 'nodes_'
            for i in self.graphNodes:
                string+= '|'+str(i)
            return string
        if self.distinguishables is not None:
            string = 'dists_'
            for i in self.distinguishables:
                string+= '|'+str(i)
            return string

    def addIndividual(self, map, ind) :
        """
        add an individual into the fragment based on its mappings
        """
        # if non-parametric
        if self.distinguishables is not None:
            element =  map.distinguish(ind)
            if element is not None:
                if element not in self.distinguishables:
                    self.distinguishables.append(element)
        # if parametric
        elif self.graphNodes is not None:
            element =  map.distinguish(ind)
            if element is not None:
                if element not in self.graphNodes:
                    self.graphNodes.append(element)
        # if description
        elif ind.type == 'ind-description':
            check = False
            for i in range(len(self.individuals)):
                if self.individuals[i].similarFragment(ind):
                    check = True
                    self.individuals[i].mergeFragment(ind)
            if not check:
                self.individuals.append(ind)
        # everything else
        else:
            self.individuals.append(ind)
            
    def mapping(self):
        return self.sort.baseSort().mapping()
    
    def dimensionality(self):
        return self.sort.baseSort().dimension()
    
    def __iter__(self):
        if self.distinguishables is not None:
            return fragment.distIter(self.distinguishables)
        elif self.graphNodes is not None:
            return iter(self.graphNodes)
        else:
            return iter(self.individuals)
            
    class distIter(object):
        def __init__(self, dists):
            self.i = 0
            self.dists = dists
        
        def __iter__(self):
            return self
        
        def __next__(self):
            if self.i < len(self.dists):
                self.i += 1
                return self.dists[self.i-1].element.individual
            else:
                raise StopIteration

    class nodeIter(object):
        def __init__(self, nodes):
            self.i = 0
            self.nodes = nodes
        
        def __iter__(self):
            return self
        
        def __next__(self):
            if self.i < len(self.nodes):
                self.i += 1
                return self.nodes[self.i-1].individual
            else:
                raise StopIteration

