# -*- coding: utf-8 -*-

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

from builtins import object
class ruleApplication(object):

    """
    Rule application class.

    The rule applications are created during the detection process and saved in a list.
    The list would be returned by the end of the process. The rule applications can then
    be used to perform transformation on the detected shape.

    **Object Parameters:** 

    * **rule** (:class:`~sortal.grammar.rules.rule.rule`) -- the rule this rule application belongs to.
    * **transformSet** (:class:`~sortal.transform.transformSet.transformSet`) -- the geometric transformation set.
    * **paramSet** (:class:`~sortal.transform.paramSet.paramSet`) -- the description transformation.
    * **matchForm** (:class:`~sortal.form.form.form`) -- the transformed left hand side form.
    * **resultForm** (:class:`~sortal.form.form.form`) -- the transformed right hand side form.
    """

    count = 0
    def __init__(self, rule, tf, ps, match, result):
        self.type = 'grammar-ruleApplication' 
        self.rule = rule
        self.transformSet = tf
        self.paramSet = ps
        self.matchForm = match
        self.resultForm = result

        if ps is None:
            self.removedForm = rule.applhs.edit(tf)
            self.addForm = rule.apprhs.edit(tf)
        else:
            self.removedForm = rule.applhs.edit(tf).edit(ps)
            self.addForm = rule.apprhs.edit(tf).edit(ps)
        
    def applicable(self, form):
        """
        check if the ruleApplication is applicable to a form
        """
        return self.matchForm.partOf(form)
        
    def __eq__(self, other):
        if other is None or other.type != 'grammar-ruleApplication':
            return False        
        return (self.rule == other.rule and 
                self.matchForm == other.matchForm and
                self.resultForm == other.resultForm)

    def clearTags(self, element):
        if 'form' in element.type:
            for e in element:
                self.clearTags(e)
        else:
            element.key = None
            if element.hasAttribute():
                self.clearTags(element.attribute)

    def perform(self, form):
        """
        apply an application on a form
        """
        form.difference(self.matchForm)
        form.sum(self.resultForm.duplicate())
        ruleApplication.count +=1
        self.clearTags(form)
        return form
    
        