# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import range
from builtins import super
from builtins import str
from builtins import object

from copy import deepcopy

class flowApplication(object):
    """
    flowApplication: an object containing the result of a flow application

    **Object Parameters:**

    * **name** (str) -- the name of the flow.
    * **description** (str) -- the description of the flow.
    * **printProcess** (boolean) -- the name of the flow.
    * **random** (boolean) -- the description of the flow.
    * **path** (list of string) -- the list of rule name and application index indicating the path taken for the flow application result.
    * **pathString** (string) -- string indicating the path taken for the flow application result (only available after the finalizeResult function is called).
    * **result** (~sortal.form.form.form) -- the final result of the flow application (only available after the finalizeResult function is called).
    """

    type = 'rule-flowApplication'
    
    def __init__(self, printProcess = True, random = True, *args, **kwargs):
        """
        Initialize a line-segment using its two end-points.
        The lower end-point will be saved as self.start; the higher end-point will be saved as self.end.

        :param printProcess: whether the process of resolving flow will be printed out in string format for debugging
        :type printProcess: boolean
        :param random: whether the selection of applications are chosen at random
        :type random: boolean
        """
        self.printProcess = printProcess
        self.random = random
        self.backtrace = False

        self.path = []
        self.pathString = ''

        self.result = None
        self.resultForms = []

        self.rulingTraces = {}
        self.rulingCycles = {}

    def __str__(self):
        return self.pathString

    def addPath(self, string):
        """
        add a string to the path
        """
        self.path.append(string)

    def removePath(self):
        """
        remove the latest rule in the path
        """
        self.path.pop()
        self.resultForms.pop()

    def finalizeResult(self):
        """
        finalize the application result, generating pathString and result form
        """
        self.pathString = '<Begin>'
        for i in self.path:
            self.pathString += i
        self.pathString += ' <End>'
        if len(self.resultForms) > 0:
            self.result = self.resultForms[-1].duplicate()

    def addResult(self, form):
        """
        add a result form
        """
        self.resultForms.append(form.duplicate())

    def getRulingTrace(self, rulingString):
        """
        get the trace of a ruling
        """
        if rulingString not in self.rulingTraces:
            self.rulingTraces[rulingString] = None
        return self.rulingTraces[rulingString]

    def setRulingTrace(self, rulingString, rulingTrace):
        """
        set the trace of a ruling
        """
        self.rulingTraces[rulingString] = rulingTrace

    def getRulingCycle(self, rulingString):
        """
        get the trace of a ruling
        """
        if rulingString not in self.rulingCycles:
            self.rulingCycles[rulingString] = []
        return self.rulingCycles[rulingString]

    def duplicate(self):
        return deepcopy(self)

    def updateApp(self, other, rule):
        """
        update the application with the path, result and cycle of a specific rule from another app
        """
        self.path = other.path
        self.resultForms = other.resultForms
        self.rulingCycles[rule] = other.rulingCycles[rule]

