# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import range
from builtins import super
from builtins import str

import sys

from .ruling import ruling
from .flowApplication import flowApplication
from copy import copy

class ruleNegation(ruling):
    """
    ruleNegation: !r -- performing the ruling in the negation, if the ruling fails, continue on, 
    backtrack otherwise. There should be no quantifier and matching declared for ruleNegation

    **Object Parameters:** 

    * **rule** (:class:`~sortal.grammar.rules.ruling.ruling`) -- the ruling object contained in the ruleNegation.

    """

    type = 'rule-negation'
    
    def __init__(self, rule, *args, **kwargs):
        super(ruleNegation, self).__init__()        
        self.rule = rule
        
    def __str__(self):
        return self.toString()

    def toString(self, full = False):
        if self.rule.type == 'rule':
            return '!'+self.rule.name
        return '!'+ self.rule.toString(full)

    def setPo(self):
        return

    def apply(self, form, flowApp = None, backtrace = False, *args, **kwargs):
        """
        applying the rule negation onto a shape form

        :param form: the form to apply the ruling on; this form will be modified to 
            become the resulting form at the end of this method
        :type form: ~sortal.form.form.form
        :param flowApp: the flow application indicating the characteristic of the application 
        :type flowApp: ~sortal.grammar.rules.flowApplication.flowApplication
        :param backtrace: indicate if this is a backtrace from a flow
        :type backtrace: boolean
        :returns: the resulting form after applying the ruling; 
            the input form is also modified to become the resulting form
        """

        # if backtrace then the negation is just false
        if backtrace:
            return False

        if flowApp is None:
            flowApp = flowApplication()

        f = form.duplicate()
        if flowApp.printProcess: 
            print()
            for i in flowApp.path:
                print(i, end='')
            print(' --> Neg(', end='')
        sys.stdout.flush()

        # try to apply the rule
        try:
            dummyFlowApp = flowApplication(flowApp.printProcess, flowApp.random)
            dummyFlowApp.path = ['\t__Neg:(']
            app = self.rule.apply(f, dummyFlowApp, backtrace = backtrace)
        except:
            if flowApp.printProcess:
                if self.rule.type == 'rule':
                    print(' error: ##'+self.rule.name+'## ', end='')
                else:
                    print(' error: ##'+str(self.rule)+'## ', end='')
            app = True
        # if the rule is applicable then the negation is false
        if app != False:
            if flowApp.printProcess: print(')', end = '')
            return False
        # otherwise the negation is true
        if flowApp.printProcess: print(')\n--Negation satisfied--', end='')
        return form


    def manualApply(self, form, flowApp = None):
        if flowApp is None:
            flowApp = flowApplication()
        print('checking:', self)
        check = self.apply(form, flowApp)
        if check is False:
            print('--Negation Failed--')
            return False
        else:
            print('--Negation Satisfied--')
        return form

    def manualApplyPeek(self, form, flowApp = None):
        """
        Get all the possible first step applications of the ruling on the form based on the flowApp. For ruleNegation,
        it would only check if the contained ruling cannot be executed and continue on

        :param form: the form to apply the ruling on; this form will be modified to 
            become the resulting form at the end of this method
        :type form: ~sortal.form.form.form
        :param flowApp: the flow application indicating the characteristic of the application 
        :type flowApp: ~sortal.grammar.rules.flowApplication.flowApplication
        :param newInvoke: indicate if this is a new invocation of the ruling (i.e. not continuing from previous manualApplyPeek)
        :type newInvoke: boolean
        :returns: a list of two values: 
            first value is a list of flowApplication objects that are the different possible step results of the flow, 
            second value is a boolean indicating whether the ruling is already satisfied -> can continue on to the next ruling
            or end the flow if it's the topmost ruling in the flow
        """
        if flowApp is None:
            flowApp = flowApplication()
        check = self.apply(form, flowApp)
        if check is False:
            return [[], False]
        return [[], True]

    def setPoCheck(self, check = None):
        self.possessiveCheck = check
        if check is True:
            check = False
        if self.rule.type != 'rule':
            self.rule.setPoCheck(check)
