# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import range
from builtins import super
from builtins import str
from builtins import object

import sys

class ruling(object):
    """
    abstract class for rule and flow
    """
    type = 'rule-ruling'

    GREEDY = 'G'
    LAZY = 'L'
    POSSESSIVE = 'PO'
    PROBABILISTIC = 'PR'
    
    matchingDict = {'G': '', 'L': '?', 'PO': '+', 'PR': '*'}
    
    def __init__(self):
        self.trace = None
        self.possessiveCheck = None
        self.quantifier = None
        self.matching = None
        self.rules = None

    def __str__(self):
        return self.toString()

    def toString(self, full = False):
        s = ''
        printSpace = False
        for i in self.rules:
            if printSpace:
                s+= ' '
            else:
                printSpace = True
            if i.type == 'rule':
                s+=i.name
            else:
                s+=i.toString(full)
        if self.matching == 'G':
            if self.type == 'rule-sequence':
                if not full and len(self.rules) == 1:
                    return s + self.quantifierString() 
                return '(' + s + ')' + self.quantifierString()
            elif self.type == 'rule-disjunction':
                return '[' + s + ']' + self.quantifierString()
            elif self.type == 'rule-disjunctionP':
                return '[*' + s + ']' + self.quantifierString()
        else:
            qS = self.quantifierString() 
            if qS == '':
                qS = '{1}'
            if self.type == 'rule-sequence':
                if not full and len(self.rules) == 1:
                    return s + qS + self.matchingDict[self.matching]
                return '(' + s + ')' + qS + self.matchingDict[self.matching]
            elif self.type == 'rule-disjunction':
                return '[' + s + ']' + qS + self.matchingDict[self.matching]
            elif self.type == 'rule-disjunctionP':
                return '[*' + s + ']' + qS + self.matchingDict[self.matching]
    
    def quantifierString(self):
        """
        get the appropriate quantifier string of the ruling
        """
        if self.quantifier == [1,1]:
            return ''
        elif self.quantifier == [0,1]:
            return '?'
        elif self.quantifier == [0,None]:
            return '*'
        elif self.quantifier == [1,None]:
            return '+'
        elif self.quantifier[1] is None:
            return '{'+str(self.quantifier[0])+',}'
        elif self.quantifier[0] == self.quantifier[1]:
            return '{'+str(self.quantifier[0])+'}'
        else:
            return '{'+str(self.quantifier[0])+','+str(self.quantifier[1])+'}'
            
    def setPo(self):
        """
        set rule as posessive
        """
        for i in range(len(self.rules)):
            if self.rules[i].type != 'rule':
                self.rules[i].matching = 'PO'
                self.rules[i].setPo()

    def setPoCheck(self, check = None):
        if self.type == 'rule':
            return
        self.possessiveCheck = check
        if check is True:
            check = False
        for i in self.rules:
            if i.type != 'rule':
                i.setPoCheck(check)

    @staticmethod
    def printS(s):
        print()
        for i in s:
            print(i, end='')
        print(' -->', end='')
        sys.stdout.flush()
    
    def __eq__(self, other):
        return str(self) == str(other)
        
    def __lt__(self, other):
        return str(self) < str(other)
        
    def __gt__(self, other):
        return str(self) > str(other)
        
    def __le__(self, other):
        return str(self) <= str(other)
        
    def __ge__(self, other):
        return str(self) >= str(other)
        
    def apply(self, *args, **kwargs):
        raise NotImplementedError

class FlowTermination(Exception):
    pass