# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import super
from builtins import str

from .individual import individual
from copy import deepcopy
import datetime


class date(individual):
    """
    date time class

    **Object Parameters:** (not including inherited attributes)

    * **value** (datetime.date) -- datetime.data object representing the day, month and year of the specified date

    **Class Properties:**

    * **Behavior**: :class:`~sortal.form.discrete.discrete`
    * **Sort Argument**: :class:`~sortal.sort.parameter.argument.argument.argumentType`.LOCALE
    * **Mapping**: :class:`~sortal.map.exactMap.exactMap`

    """

    type = 'ind-date'
    def __init__(self, sort, day = None, month = None, year = None, *args, **kwargs):
        """
        Initializing a date, if day, month and year are not specified, the current date would be used
        """
        super(date, self).__init__(sort)
        if day == None and month == None and year == None:
            self.value = datetime.date.today()
        else:
            self.value = datetime.date(year, month, day)
                
    def printVal(self):
        """
        print out a string representing the value of this individual, used in form.printE()
        """
        return str(self.value)            
                        
    def equals(self, other):
        """
        check equality
        """
        return (self.value == other.value)
        
    def compareValue(self, other):
        """
        compare date
        """
        if self.value > other.value:
            return 1
        elif self.value == other.value:
            return 0
        return -1    
                
    def replicateInd(self):
        """
        Create a duplicate of the individual without attributes
        
        :returns: the duplicate object
        """
        return date(self.sort, self.value.day, self.value.month, self.value.year)
