# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import str

from .tree import tree
from .symbols import symbols
from .numberLiteral import numberLiteral
from ...struct.coordinate import coordinate, pi


class literal(tree):
    """
    predefined literal: nil, true, false, pi, e
    """
    keywords = {}
    operators = {}

    type = 'description-literal'
    def __init__(self,name):
        name = str(name).lower()
        self.name = name
        self.value = None
        if name == 'pi':
            self.value = coordinate(pi)
        if len(name) == 1:
            literal.operators[name] = self
        else:
            literal.keywords[name] = self
            
            
    @staticmethod
    def isLiteral(name):
        """
        check if a string is a literal name
        """
        name = name.lower()
        if len(name) == 1:
            return name in literal.operators
        return name in literal.keywords
        

    @staticmethod
    def createLiteral(name):
        """
        create a literal
        """
        name = name.lower()
        if len(name) == 1:
            if name in literal.operators:
                return literal.operators[name]
        elif name in literal.keywords:
            return literal.keywords[name]
        return literal(name)
        
    @staticmethod
    def getLiteral(name):
        """
        get an existing literal with its name
        """
        name = name.lower()
        if len(name) == 1:
            return literal.operators[name]
        return literal.keywords[name]
        
    def getChar(self):
        return self.name[0]
        
    def __eq__(self, other):
        if not isinstance(other, literal):
            return False
        return self.name == other.name
        
    def __lt__(self, other):
        return self.compareValue(other) == -1
        
    def __gt__(self, other):
        return self.compareValue(other) == 1
        
    def __hash__(self):
        hashnum = 3
        hashnum = 59 * hashnum + hash(self.name)
        return hashnum
    
    def compareValue(self, other):
        """
        comparing literals
        """
        if self.name == 'nil' or self.name == 'e':
            if other == literal.createLiteral('nil') or other == literal.createLiteral('e'):
                return 0
            if other.type == 'description-numberLiteral' and other.value == 0:
                return 0            
            if other.type == 'description-stringLiteral' and other.s == '':
                return 0
            if other.type == 'description-tuple' and other.isEmpty():
                return 0
            else:
                return -1
        elif self.name == 'pi':
            if other == literal.createLiteral('nil') or other == literal.createLiteral('e'):
                return 1
            if other == literal.createLiteral('pi'):
                return 0
            if other.type == 'description-numberLiteral':
                if pi < other.value:
                    return -1
                if pi > other.value:
                    return 1
                return 0         
            return -1
        elif self.name == 'true':
            if other == literal.createLiteral('true'):
                return 0
            if other.type == 'description-numberLiteral' and other.value == 1: # or other.value != 0 ?
                return 0
        elif self.name == 'false':
            if other == literal.createLiteral('false'):
                return 0
            if other.type == 'description-numberLiteral' and other.value == 0:
                return 0
        elif other.type == 'description-literal' :
            if self.name > other.name:
                return 1
            if self.name < other.name:
                return -1
            return 0
        elif other.type == 'description-stringExpression':
            return -other.compareValue(self)
        else:
            c = hash(self.__class__) - hash(other.__class__)
        if c < 0:
            return -1
        if c > 0:
            return 1
        return 0
        
    def __str__(self):
        return self.name
        
    def printVal(self):
        return 'lit:'+self.name
        
    def evaluate(self, assignment, sort):
        """
        evaluating the literal: the result would be itself
        """
        return self