# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import range
from builtins import super
from builtins import str


from .tree import tree
from .numberLiteral import numberLiteral
from .function import function
from ...struct.coordinate import coordinate

class stringLiteral(tree):
    """
    description component representing string values, can exist on its own as a string in the description
    or the result of string expressions.
    """
    LENGTH = 'length'
    LEFT = 'left'
    RIGHT = 'right'
    
    FUNCTION = [LENGTH,LEFT,RIGHT]
    ASSISTS = [None, numberLiteral, numberLiteral]
    

    for fname in FUNCTION:
        function.reserve(fname)

    
    type = 'description-stringLiteral'
    
    def __init__(self, s, encoded = None):
        s = str(s)
        if s[0] == '"' and s[-1] == '"':
            self.s = s[1:-1]
        else:
            self.s = s
        self.encoded = encoded
        
    def __eq__(self, other):
        if other.type != 'description-stringLiteral':
            return False
        if self.s == other.s:
            return True
        return False
        
    def __lt__(self, other):
        return self.compareValue(other) == -1
        
    def __gt__(self, other):
        return self.compareValue(other) == 1
        
    def __len__(self):
        return len(self.s)

    def compareValue(self, other):
        """
        comparing two string literals
        """
        if other.type == 'description-stringLiteral':
            if self.s > other.s:
                return 1
            elif self.s < other.s:
                return -1
            return 0
        elif other.type == 'description-stringExpression' or other.type == 'description-literal':
            return -other.compareValue(self)
        c = hash(self.__class__) - hash(other.__class__)
        if c > 0:
            return 1
        if c < 0:
            return -1
        return 0
        
    def __str__(self):
        return '"'+self.s+'"'
        
    def printVal(self):
        return 'strLit:'+self.s
        
    def apply(self, function, value = None):
        """
        resolving a stringLiteral function
        """
        if value is None:
            if function == self.LENGTH:
                return numberLiteral(coordinate(len(self.s)))
            return super(self).apply(function)
        else:
            if function == self.LEFT:
                return stringLiteral(self.s[:int(value.value)])
            elif function == self.RIGHT:
                return stringLiteral(self.s[int(value.value):])
            else:
                return super(self).apply(function)
            
    def evaluate(self, assignment, sort):
        """
        evaluating the numberLiteral: the result would be itself
        """
        return self
        
    def functionExists(self, name, assist= None):
        """
        check if there exist a function with a specific name for string literal
        """
        for i in range(len(stringLiteral.FUNCTION)):
            if stringLiteral.FUNCTION[i] == name and stringLiteral.ASSISTS[i] == assist:
                return True
        return False