# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import super

from .individual import individual
from copy import deepcopy


class imageUrl(individual):
    """
    Image URL class

    **Object Parameters:** (not including inherited attributes)

    * **value** (str) -- url of the image
    * **icon** (str) -- url of the icon

    **Class Properties:**

    * **Behavior**: :class:`~sortal.form.discrete.discrete`
    * **Sort Argument**: :class:`~sortal.sort.parameter.argument.argument.argumentType`.NONE
    * **Mapping**: :class:`~sortal.map.exactMap.exactMap`
    """
    
    type = 'ind-imageUrl'
    
    def __init__(self, sort, value = None, icon = None, *args, **kwargs):
        """
        Initialize an image Url
        """
        super(imageUrl, self).__init__(sort)
        if value == None:
            self.value = ''
            self.icon = ''
        else:
            self.value = value
            self.icon = icon
                
    def isNil(self):
        """
        check if the individual is Nil
        """
        return self.value == ''
        
    def equals(self, other):
        """
        check equality
        """
        return (self.value == other.value) and (self.icon == other.icon)
        
        
    def compareValue(self, other):
        """
        compare two individuals
        """
        if self.value > other.value:
            return 1
        if self.value < other.value:
            return -1
        if self.icon > other.icon:
            return 1
        if self.icon < other.icon:
            return -1
        return 0
        
    def replicateInd(self):
        """
        Create a duplicate of the individual without attributes
        """
        return imageUrl(self.sort, self.value)

