# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import super
from builtins import str

from .individual import individual
from copy import deepcopy
class label(individual):
    """
    A label is an alphanumerical data entity with a discrete behavior,
    i.e., the value of the sum of two labels is the collection of both labels,
    unless both labels are identical, in which case is is either label.

    The Label class extends on the Individual class.
    It defines the characteristic individual for labels.
    A label is represented as a string.

    **Object Parameters:** (not including inherited attributes)

    * **value** (str) -- content of the label.

    **Class Properties:**

    * **Behavior**: :class:`~sortal.form.discrete.discrete`
    * **Sort Argument**: :class:`~sortal.sort.parameter.argument.argument.argumentType`.NONE
    * **Mapping**: :class:`~sortal.map.exactMap.exactMap`

    """

    type = 'ind-label'
    
    def __init__(self, sort = None, s = None, *args, **kwargs):
        """
        initialize label
        label contains: type: 'ind-label', value: a string
        """
        super(label, self).__init__(sort)
        if s == None:
            self.value = ''
        else:
            self.value = s
            
    def toString(self, printKey):
        """
        convert individual to string
        """
        s = ''
        if printKey and self.key is not None:
            s += str(self.key)+' '
        return s+'"'+self.value+'"'
        
    def printVal(self):
        """
        print out a string representing the value of this individual, used in form.printE()
        """
        return str(self.value)   
         
    def length(self):
        """
        return the length of the value string
        """
        return len(self.value)
        
    def isNil(self):
        """
        check if the label is empty
        """
        return self.value == ''
        
    def equals(self, other):
        """
        check equality
        """
        return (self.value == other.value)

    def compareValue(self, other):
        """
        comparing value of 2 labels
        """
        if self.value > other.value:
            return 1
        elif self.value == other.value:
            return 0
        return -1   
        
    def replicateInd(self):
        """
        make a duplicate without attribute
        """
        return label(self.sort, self.value)

    @staticmethod
    def parse(reader, sort):
        """
        parse individual from string
        """
        ind = label(sort, reader.value()[1:-1])
        reader.newToken()
        return ind
