# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import


from builtins import super
from builtins import str

from .individual import individual
from copy import deepcopy
from ..struct.coordinate import coordinate


class numeric(individual):
    """
    Numeric class, containing a number as value

    **Object Parameters:** (not including inherited attributes)

    * **value** (:class:`~sortal.struct.coord.coordinate_float.coordinate`) -- the number the numeric object represents

    **Class Properties:**

    * **Behavior**: :class:`~sortal.form.discrete.discrete`
    * **Sort Argument**: :class:`~sortal.sort.parameter.argument.argument.argumentType`.NONE
    * **Mapping**: :class:`~sortal.map.exactMap.exactMap`
    """
    type = 'ind-numeric'

    def __init__(self, sort, value = None, *args, **kwargs):
        """
        Initializing a new numberic object
        """
        super(numeric, self).__init__(sort)
        if value is None:
            self.value = 0
        else:
            self.value = coordinate(value)
                
    def printVal(self):
        """
        print out a string representing the value of this individual, used in form.printE()
        """
        return str(self.value)   
                 
    def equals(self, other):
        """
        check equality
        """
        return (self.value == other.value)
        
    def compareValue(self, other):
        """
        compare two individuals
        """
        if self.value > other.value:
            return 1
        elif self.value == other.value:
            return 0
        return -1    
        
    def replicateInd(self):
        """
        Create a duplicate of the individual without attributes
        """
        return numeric(self.sort, self.value)
        
    @staticmethod
    def parse(reader, sort):
        """
        parse the individual
        """
        neg = False
        if reader.token() == '-':
            reader.newToken()
            neg = True
        num = reader.value()
        ind = numeric(sort, num)
        if neg:
            ind.value = - ind.value
        reader.newToken()
        return ind
        