# -*- coding: utf-8 -*-


#difference --> arithmatic difference (1.0-0.1 = 0.9)
#no weight form == zero weight form (0 is default)
#if the weight become 0 --> remove the weight but not its base (line/point)

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import super
from builtins import str

from .individual import individual


class rWeight(individual):
    """
    rWeight class. (weight for Rhinoceros)
    
    The rWeight object indicates the weight of the stroke (in float value) for the associating
    object that it is an attribute of.

    **Object Parameters:** (not including inherited attributes)
    
    * **value** (float) -- the weight value

    **Class Properties:**

    * **Behavior**: :class:`~sortal.form.ordinal.ordinal`
    * **Sort Argument**: :class:`~sortal.sort.parameter.argument.argument.argumentType`.UPPERBOUND
    * **Mapping**: :class:`~sortal.map.exactMap.exactMap`

    """

    MIN = 0
    
    type = 'ind-rWeight'
    
    def __init__(self, sort = None, w = 0, *args, **kwargs):
        """
        Initializing an weight. If the weight value is higher than the maximum weight allowed
        (declared when creating the weight sort), the weight value is set to the maximum.

        :param sort: sort of the weight
        :type sort: ~sortal.sort.sort.sort
        :param w: the weight value
        :type w: float
        """
        super(rWeight, self).__init__(sort)
        if sort.baseSort().arguments is not None and w > sort.baseSort().arguments.value:
            w = sort.baseSort().arguments.value
        self.value = w
            
    def printVal(self):
        """
        Print out a string representing the value of this individual, used in form.printE()
        """
        return str(self.value)
        
    def length(self):
        return len(self.value)
        
    def isNil(self):
        """
        Check if the weight value is at minimum
        """
        return self.value == self.MIN
        
    def equals(self, other):
        """
        check equality
        """
        return (self.value == other.value)

        
    def compareValue(self, other):
        """
        compare two weight objects
        """
        if self.value > other.value:
            return 1
        elif self.value == other.value:
            return 0
        return -1    
    
    def replicateInd(self):
        """
        Create a duplicate of the individual without attributes
        
        :returns: the duplicate object
        """
        return rWeight(self.sort, self.value)

    def getMinInd(self):
        """
        get a weight with the minimum weight value (weight = 0)
        """
        return rWeight(self.sort)
        
    def complement(self, other):
        """
        find the complement of this weight with another
        """
        if self.isNil():
            return False
        if other is None or other.isNil():
            return True
        self.value = self.value - other.value
        if self.value <= self.MIN:
            self.value = self.MIN
            return False
        return True

    @staticmethod
    def parse(reader, sort):
        """
        parse weight from a parse reader
        """
        neg = False
        if reader.token() == '-':
            reader.newToken()
            neg = True
        try:
            ind = rWeight(sort, int(reader.value()))
        except:
            ind = rWeight(sort, float(reader.value()))
        if neg:
            ind.value = -ind.value
        reader.newToken()
        return ind

