# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import str

from builtins import object
from copy import copy

class distinguishable(object):
    """
    distinguishable object class for similarity mapping

    **Object Parameters:** (not including inherited attributes)

    * **element** (:class:`~distinguishable.elm`) -- the element defining this distinguishable
    * **counterparts** (list of :class:`~distinguishable.elm`) -- list of elms that corresponds with this distinguishable's elm 

    """

    class elm(object):
        """
        distinguishable element for similarity mapping

        **Object Parameters:** (not including inherited attributes)

        * **individual** (:class:`~sortal.ind.individual.individual`) -- the codescriptor of all the individuals in the elm's forms
        * **forms** (dict of list of :class:`~sortal.ind.individual.individual` according to individual sort name) -- lists of individuals with the same codescriptor grouped base on their sort
        """
        def __init__(self, codescriptor):
            self.individual = codescriptor
            self.forms = {}
            
        def __repr__(self):
            return str(self)
            
        def __str__(self):
            return str(self.individual)+str(self.forms)
            
        def add(self, ind):
            """
            add an individual into the forms of the elm
            """
            sort = ind.sort
            check = False
            for i in self.forms.keys():
                if i == sort.name:
                    form = self.forms[sort.name]
                    check = True
            if check == False:
                form = []
                self.forms[sort.name] = form
            form.append(ind)
            
        def __eq__(self, other):
            if not isinstance(other, distinguishable.elm):
                return False
            return self.individual == other.individual
            
    def __init__(self, ind):
        self.element = distinguishable.elm(ind)
        self.counterparts = []
        
    def __repr__(self):
        return str(self)
            
    def __str__(self):
        return str(self.element)+'|'+str(self.counterparts)

    def addCounterpart(self, ind):
        """
        add an individual as a counterpart for this distinguishable
        """
        element = ind.codescriptor()
        counterpart = None
        for i in self.counterparts:
            if i.individual == element:
                counterpart = i
        if counterpart is None:
            counterpart = distinguishable.elm(element)
            self.counterparts.append(counterpart)
        counterpart.add(ind)
        
    def getCounterparts(self):
        """
        get the counterpart distinguishables
        """
        return copy(self.counterparts)
            
    def addInstance(self, ind):
        """
        add an individual into the distinguishable's elm
        """
        self.element.add(ind)
    
    def clear(self):
        """
        clear the counterparts of this distinguishable
        """
        self.counterparts= []
        

