# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from .mapping import mapping
from ..transform.transform import transform

class exactMap(mapping):
    """
    Exact mapping class.

    The exact mapping maps only objects that are exactly the same without any transformation
    (the transformation given by the mapping is an empty transformation). Exact mappping is 
    only used if there is no similarity mapping or graph mapping defined (no geometric object in the
    lhs of the rule which includes points, lines, line segments, plane, plane segments and curves)
    """
    type = 'map-individual-exactMap'
    
    def __init__(self):
        self.transforms = transform()
        return
        
    @staticmethod
    def getInstance():
        """
        get a new exact mapping instance
        """
        return exactMap()
        
    def duplicate(self):
        return self
    
    def clear(self):
        """
        clear the previous detection data
        """
        return
    
    def map(self, fromInd, toInd):
        """
        Try to map an object from the shape to a lhs object and by checking their equality
        
        :param fromInd: lhs object
        :type fromInd: ~sortal.ind.individual.individual
        :param toInd: shape object to be mapped
        :type toInd: ~sortal.ind.individual.individual
        :returns: boolean -- whether the two objects can be mapped
        """
        return fromInd == toInd
        
    def potentialTransforms(self):
        """
        Find all potential transformations for the mapping: one empty transformation
        """
        return [self.transforms]
        