# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import object
from enum import Enum

class mapping(object):
    """
    A *mapping* specifies the conditions under which an
    :class:`~sortal.ind.individual.individual` can map onto another individual of the same
    :class:`~sortal.sort.sort.sort`. For instance, a label maps onto another label if it
    has the same string value; a line segment maps onto another line segment if
    it is embedded in the other segment under a similarity transformation.
    Various mappings can apply to the same sort. For example, a label may
    alternatively map onto another label under an uppercase-lowercase
    transformation.

    Each characteristic :class:`~sortal.ind.individual.individual` defines a default mapping.

    The *mapping* class defines the constants specifying the different types
    of mapping. Each mapping type (instance) defines a degrees-of-freedom. This
    value is used to order mappings and, more importantly, (component) sorts when
    determining a mapping for composite sorts. Other constants serve to express
    the dimensionality of the individuals of a sort. The dimensionality of
    individuals of a sort actually primarily relates to the operational behavior
    of the individuals. Under a discrete operational behavior
    (:class:`~sortal.form.discrete.discrete`), two individuals combine only if they are
    identical. Their dimensionality is identified as 0. Under an interval or
    ordinal operational behavior (:class:`~sortal.form.interval.interval`,
    :class:`~sortal.form.ordinal.ordinal`), two individuals combine if they overlap
    within their linear dimension. Their dimensionality is identified as 1.
    Under an an areal operational behavior (:class:`~sortal.form.areal.areal`), two
    individuals combine they overlap within their respective carrier. Their
    dimensionality is either 2 or 3 according to their carrier's dimension.
    """
    types = {}
    
    DISCRETE = 1
    DISCRETE_DESCRIPTION = 1.1
    DISCRETE_GEOMETRY = 1.2
    DISCRETE_PARAMETRIC_GEOMETRY = 1.3

    CYCLIC = 2
    PARAMETRIC_CYCLIC = 2.1
    LINEAR = 2.2
    PARAMETRIC_LINEAR = 2.3  

    PLANAR = 3
    PARAMETRIC_PLANAR = 3.1
    
    SPATIAL = 4
    
    def __init__(self):
        self.id = None
    
    def isValid(self):
        return True
        
    def dof(self, type = None):
        if type is None:
            return mapping.types[self.__class__]
        return mapping.types[type]
        
    @staticmethod
    def register(type, dof):
        mapping.types[type] = dof
        
    @staticmethod
    def getInstance(self):
        raise NotImplementedError 
        
    def clear(self):
        raise NotImplementedError 
    
    def map(self, *args, **kwargs):
        raise NotImplementedError 
    
    def potentialTransforms(self):
        raise NotImplementedError

    def analyzeRHS(self, rhs):
        return

    def postAnalysis(self):
        return
