# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from .mapping import mapping
from ..transform.paramSet import paramSet
from ..transform.transform import transform
from ..ind.description import description
from ..sort.sort import sort as sortClass

class parametricMap(mapping):
    """
    mapping class for description
    """

    type = 'map-parametric'
    def __init__(self, *args, **kwargs):
        super(parametricMap, self).__init__()
        self.paramsets = {}
        self.predicates = []
        self.postCheck = False
        
    @staticmethod
    def getInstance():
        """
        Get a new parametricMap instance
        """
        return parametricMap()

    def isValid(self):
        """
        Check if the mapping is valid
        """
        return True

    def clear(self):
        """
        Clear the previous detection data
        """
        self.paramsets = {}
    
    def map(self, fromInd, toInd):
        """
        Mapping a shape description to a lhs description for detection
        """
        params = paramSet()
        if fromInd.map(params, toInd):
            if not params.isEmpty():
                key = fromInd.sort.name+"|"+str(fromInd)
                try:
                    array = self.paramsets[key]
                except KeyError:
                    array = []
                    self.paramsets[key] = array
                if params not in array:
                    array.append(params)
            return True
        return False
        
    def potentialParamSets(self):
        """
        get all potential description transformations
        """
        result = []
        if len(self.paramsets) == 0:
            return result
        for string in self.paramsets:
            array = self.paramsets[string]
            if len(result) == 0:
                for i in array:
                    result.append(i)
            else:
                res = []
                
                for i in result:
                    for j in array:
                        pSet = paramSet()
                        pSet.merge(i)
                        pSet.merge(j)
                        res.append(pSet)
                result = res
        return result
        
    def potentialTransforms(self):
        """
        find all potential geometric transformations: one empty transformation
        """
        return [transform()]
        
    def addPredicate(self, cond, keyList):
        """
        add predicate to the mapping (only work if the rule does not have a graphMaps)
        """
        cond = cond.lower()
        self.predicates.append([cond,keyList])

    def clearPredicates(self):
        """
        clear the predicate list
        """
        self.predicates = []

    def evaluatePredicate(self, tlhs, param):
        """
        resolve the mapping predicates: only description predicates are resolved
        """
        param.lhs = tlhs
        self.postCheck = True
        for cond in self.predicates:
            if cond[0] == 'description':
                for keyI in cond[1]:
                    val = description(sortClass.getSort('__dummyDesc__'), keyI[0])
                    try:
                        val = val.evaluateInd(param)
                    except ValueError:
                        self.postCheck = False
                        return False
                    if keyI[1] == '{':
                        check = False
                        for comp in keyI[-1]:
                            
                            comparand = description(sortClass.getSort('__dummyDesc__'), comp)
                            comparand = comparand.evaluateInd(param)
                            if comparand == val:
                                check = True
                                break
                    elif keyI[1] == '[':
                        if len(keyI[-1]) != 2:
                            raise ValueError('[ ] condition only accepts 2 arguments as min and max. e.g. [1,3] --> value between 1 and 3')
                        comp1 = description(sortClass.getSort('__dummyDesc__'), keyI[-1][0])
                        comp1 = comp1.evaluateInd(param)
                        comp2 = description(sortClass.getSort('__dummyDesc__'), keyI[-1][-1])
                        comp2 = comp2.evaluateInd(param)
                        if comp1 < val < comp2:
                            check = True
                        else:
                            check = False
                    else:
                        comparand = description(sortClass.getSort('__dummyDesc__'), keyI[-1])
                        comparand = comparand.evaluateInd(param)
                        if keyI[1] == '=':
                            check = val == comparand
                        if keyI[1] == '<':
                            check = val < comparand
                        if keyI[1] == '>':
                            check = val > comparand
                        if keyI[1] == '<=':
                            check = val <= comparand
                        if keyI[1] == '>=':
                            check = val >= comparand
                        if keyI[1] == '<>':
                            check = not (val == comparand)
                    if not check:
                        self.postCheck = False
            if self.postCheck is False:
                return False
        return True
