# -*- coding: utf-8 -*-

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

from builtins import range

from builtins import object
class classes(object):
    """
    classification result of two coplanar coForms of plane-segment.

    the classes object contains 6 lists of :class:`sortal.struct.planeStruct.bSegment`:
    
    * classes_obj.INNER_1: the list of segments of the plane-segment in coform 1 that is inside the plane segment in coform 2
    * classes_obj.OUTER_1: the list of segments of the plane-segment in coform 1 that is outside the plane segment in coform 2
    * classes_obj.INNER_2: the list of segments of the plane-segment in coform 2 that is inside the plane segment in coform 1
    * classes_obj.OUTER_2: the list of segments of the plane-segment in coform 2 that is outside the plane segment in coform 1
    * classes_obj.SAME_SHARED: the list of segments that are shared by the plane-segments in the two coform where these plane segments are on the same side with regard to the shared segment
    * classes_obj.OPP_SHARED: the list of segments that are shared by the plane-segments in the two coform where these plane segments are on the opposite sides of the shared segment
    """
    INNER_1 = 0
    
    OUTER_1 = 1
    
    INNER_2 = 2
    
    OUTER_2 = 3
    
    SAME_SHARED = 4
    
    OPP_SHARED = 5
    
    CLASS_COUNT = 6
    
    def __init__(self):
        self.classes = [None for i in range(classes.CLASS_COUNT)]
        for i in range(classes.CLASS_COUNT):
            self.classes[i] = []
            
    def printClasses(self):
        """
        print the classification result for debugging
        """
        for i in range(6):
            if i == 0:
                print('INNER_1:')
            elif i == 1:
                print('OUTER_1:')
            elif i == 2:
                print('INNER_2:')
            elif i == 3:
                print('OUTER_2:')
            elif i == 4:
                print('SAME_SHARED:')
            elif i == 5:
                print('OPP_SHARED:')
            for j in self.classes[i]:
                print('  ', j.printRhino())
            
    def addInnerOne(self, segment):
        """
        add a segment as inner_1
        """
        self.classes[classes.INNER_1].append(segment)

    def addInnerTwo(self, segment):
        """
        add a segment as inner_2
        """
        self.classes[classes.INNER_2].append(segment)
    
    def addOuterOne(self, segment):
        """
        add a segment as outer_1
        """
        self.classes[classes.OUTER_1].append(segment)
    
    def addOuterTwo(self, segment):
        """
        add a segment as outer_2
        """
        self.classes[classes.OUTER_2].append(segment)
    
    def addSameShared(self, segment):
        """
        add a segment as same_shared
        """
        self.classes[classes.SAME_SHARED].append(segment)
    
    def addOppShared(self, segment):
        """
        add a segment as opp_shared
        """
        self.classes[classes.OPP_SHARED].append(segment)
    
    def contains(self):
        """
        check if coform1 contains coform2 (empty inner_1, outer_2 and opp_shared)
        """
        return (len(self.classes[classes.INNER_1]) == 0 and
                len(self.classes[classes.OUTER_2]) == 0 and
                len(self.classes[classes.OPP_SHARED]) == 0)
        
    
    def isContained(self):
        """
        check if coform1 is contained by coform2 (empty inner_2, outer_1 and opp_shared)
        """
        return (len(self.classes[classes.INNER_2]) == 0 and
                len(self.classes[classes.OUTER_1]) == 0 and
                len(self.classes[classes.OPP_SHARED]) == 0)

    def touches(self):
        """
        check if the two coforms touch  (empty inner_1, inner_2 and same_shared; opp_shared not empty)
        """
        return (len(self.classes[classes.INNER_1]) == 0 and
                len(self.classes[classes.INNER_2]) == 0 and
                len(self.classes[classes.SAME_SHARED]) == 0 and
                len(self.classes[classes.OPP_SHARED]) != 0)
    
    def disjoint(self):
        """
        check if the two coforms are disjoint (empty inner_1, inner_2, same_shared and opp_shared)
        """
        return (len(self.classes[classes.INNER_1]) == 0 and
                len(self.classes[classes.INNER_2]) == 0 and
                len(self.classes[classes.SAME_SHARED]) == 0 and
                len(self.classes[classes.OPP_SHARED]) == 0)
    
    def aligns(self):
        """
        check if the two coforms align (empty opp_shared, same_shared not empty)
        """
        return (len(self.classes[classes.SAME_SHARED]) != 0 and
                len(self.classes[classes.OPP_SHARED]) == 0)
    
    def sumSet(self):
        """
        return a list of bSegments for constructing the sum result of the two coforms
        """
        result = []
        result += self.classes[classes.OUTER_1]
        result += self.classes[classes.OUTER_2]
        result += self.classes[classes.SAME_SHARED]
        return result
    
    def productSet(self):
        """
        return a list of bSegments for constructing the product result of the two coforms
        """
        result = []
        result += self.classes[classes.INNER_1]
        result += self.classes[classes.INNER_2]
        result += self.classes[classes.SAME_SHARED]
        return result
    
    def diffSet(self):
        """
        return a list of bSegments for constructing the difference result of the two coforms (coform1 - coform2)
        """
        result = []
        result += self.classes[classes.OUTER_1]
        result += self.classes[classes.INNER_2]
        result += self.classes[classes.OPP_SHARED]
        return result
    
    def otherDiffSet(self):
        """
        return a list of bSegments for constructing the difference result of the two coforms (coform2 - coform1)
        """
        result = []
        result += self.classes[classes.OUTER_2]
        result += self.classes[classes.INNER_1]
        result += self.classes[classes.OPP_SHARED]
        return result
    
    def symDiffSet(self):
        """
        return a list of bSegments for constructing the symdifference result of the two coforms ((coform1 - coform2) + (coform2 - coform1))
        """
        result = []
        result += self.classes[classes.INNER_1]
        result += self.classes[classes.INNER_2]
        result += self.classes[classes.OUTER_1]
        result += self.classes[classes.OUTER_2]
        return result
        
        
class lineClasses(object):
    """
    classification result of a coForm of plane-segment with a list of bSegments.

    the classes object contains 3 lists of :class:`sortal.struct.planeStruct.bSegment`:
    
    * classes_obj.INNER: the list of bsegments that are inside the coform
    * classes_obj.OUTER: the list of bsegments that are outside the coform
    * classes_obj.SHARED: the list of bsegments that are on the plane-segment in the coform
    """
    INNER = 0
    
    OUTER = 1
    
    SHARED = 2
    
    CLASS_COUNT = 3
    
    def __init__(self):
        self.classes = [None for i in range(lineClasses.CLASS_COUNT)]
        for i in range(lineClasses.CLASS_COUNT):
            self.classes[i] = []
            
    def printClasses(self):
        """
        print the classification result for debugging
        """
        for i in range(len(self.classes)):
            if i == 0:
                print('INNER:')
            elif i == 1:
                print('OUTER:')
            elif i == 2:
                print('TOUCHES:')
            for j in self.classes[i]:
                print('--', j)
            
            
    def addInner(self, segment):
        """
        add a segment to inner
        """
        self.classes[lineClasses.INNER].append(segment)

    def addOuter(self, segment):
        """
        add a segment to outer
        """
        self.classes[lineClasses.OUTER].append(segment)
    
    def addShared(self, segment):
        """
        add a segment to shared
        """
        self.classes[lineClasses.SHARED].append(segment)
    
    def isContained(self):
        """
        check if all the bSegments are contained by the coform
        """
        return len(self.classes[lineClasses.OUTER]) == 0
        
    def notInside(self):
        """
        check if all the bSegments are not inside the coform (can be on the plane boundaries)
        """
        return len(self.classes[lineClasses.INNER]) == 0
        
    def isInner(self):
        """
        check if all the bSegments are inside the coform (and not on the plane boundaries)
        """
        return (len(self.classes[lineClasses.SHARED]) == 0 and 
                len(self.classes[lineClasses.OUTER]) == 0)
        
    def touches(self):
        """
        check if all the bSegments are on the plane boundaries
        """
        return (len(self.classes[lineClasses.INNER]) == 0 and 
                len(self.classes[lineClasses.OUTER]) == 0)
    
    def disjoint(self):
        """
        check if all the bSegments are outside the coform (and not on the plane boundaries)
        """
        return (len(self.classes[lineClasses.INNER]) == 0 and
                len(self.classes[lineClasses.SHARED]) == 0)
                
    def innerSet(self):
        """
        return a list of bSegments that are inside the coform (inner)
        """
        return self.classes[lineClasses.INNER]
        
    def sharedSet(self):
        """
        return a list of bSegments that are on the plane boundaries (shared)
        """
        return self.classes[lineClasses.SHARED]
        
    def outerSet(self):
        """
        return a list of bSegments that are outside the coform (outer)
        """
        return self.classes[lineClasses.OUTER]
        
    def containedSet(self):
        """
        return a list of bSegments that contained by the coform (inner and shared)
        """
        return self.classes[lineClasses.INNER] + self.classes[lineClasses.SHARED]
        
    def notInnerSet(self):
        """
        return a list of bSegments that are not inner (outer and shared)
        """
        return self.classes[lineClasses.OUTER] + self.classes[lineClasses.SHARED]

    def notSharedSet(self):
        """
        return a list of bSegments that are not shared (outer and inner)
        """
        return self.classes[lineClasses.OUTER] + self.classes[lineClasses.INNER]