# -*- coding: utf-8 -*-

from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import with_statement

from builtins import int
from builtins import str

from builtins import object
from math import sqrt, pi, sin, cos, tan, asin, acos, atan, atan2
from math import log10, floor, ceil




class coordinate(object):
    """
    Coordinate (float) is the representational unit for geometric shape values.
    Coordinate (float) is saved and represented as python float.
    """

    """
    The precision of the floating point conversion is set to 8.

    i.e. the value of the coordinate is rounded to 8 decimal places when doing comparison
    """

    p = 8
    defaultP = 8
    tol = 10 ** -8
    negtol = -10 ** -8

    c0 = 0
    c1 = 1
    type = 'struct-coordinate'
    
    def __init__(self, num):
        """
        Initializing the coordinate.

        :param num: the value of the new coordinate
        :type num: str or int or float
        """
        if isinstance(num, coordinate):
            # raise ValueError('initiating an already existing coordinate object')
            self.value = num.value
        elif isinstance(num, int) or isinstance(num, float):
            self.value = num
        else:
            try:
                self.value = float(num)
            except:
                raise ValueError('Cannot create a coordinate with '+str(num))
                # print('Cannot create a coordinate with '+str(num))
            
    
    def __float__(self):
        """
        convert the coordinate to float
        """
        return float(self.value)
        
    def __int__(self):
        """
        convert the coordinate to int
        """
        return int(self.value)
        
    def __repr__(self):
        """
        string representation of the coordinate
        """
        return str(self.value)
        # return str(round(self.value,3))
        
    def __str__(self):
        """
        string representation of the coordinate
        """
        return str(self.value)
        # return str(round(self.value,coordinate.p))
        
    @staticmethod
    def precision():
        """
        get the current precision setting
        """
        return coordinate.p
        
    @staticmethod
    def setPrecision(prec = None):
        """
        set the current precision setting
        """
        if prec is None:
            coordinate.p = coordinate.defaultP
            coordinate.tol = 10 ** (0 - coordinate.p)
            coordinate.negtol = - 10 ** (0 - coordinate.p)
        else:
            coordinate.p = prec
            coordinate.tol = 10 ** (0 - prec)
            coordinate.negtol = - 10 ** (0 - prec)

    @staticmethod
    def setDefaultPrecision():
        """
        set the current precision setting back to default
        """
        coordinate.p = coordinate.defaultP

    @staticmethod
    def isclose(a, b):
        return abs(a-b) <= coordinate.tol * (10 ** ceil(log10(max(abs(a), abs(b)) + 1)))

        
    def __round__(self, n = None):
        """
        round the coodinate to the closest interger (or a specified decimal place)
        """
        if n is None:
            return coordinate(round(self.value))
        return coordinate(round(self.value,n))
                
    def ceil(self):
        """
        round the coodinate up to the closest interger
        """
        return coordinate(ceil(self.value))
                
    def floor(self):
        """
        round the coodinate down to the closest interger
        """
        return coordinate(floor(self.value))
                
        
    def sign(self):
        """
        find the sign of the coordinate: 1 if positive, -1 if negative, 0 if 0
        """
        if self == 0:
            return 0
        if self > 0:
            return 1
        return -1
        
    def isZero(self):
        """
        check if the coordinate is equal to 0
        """
        if self.value == 0:
            return True
        return coordinate.isclose(self.value, 0)
        # return round(self.value, coordinate.p) == 0
        
    def isOne(self):
        """
        check if the coordinate is equal to 1
        """
        if self.value == 1:
            return True
        return coordinate.isclose(self.value, 1)
        # return round(self.value, coordinate.p) == 1
    
    def sqrt(self):
        return coordinate(sqrt(self.value))
        
    def __add__(self, other):
        if not isinstance(other, coordinate):
            other = coordinate(other)
        return coordinate(self.value + other.value)   

    def __sub__(self, other):
        if not isinstance(other, coordinate):
            other = coordinate(other)
        return coordinate(self.value - other.value)

    def __mul__(self, other):
        if not isinstance(other, coordinate):
            other = coordinate(other)
        return coordinate(self.value * other.value)

    def __truediv__(self, other):
        if not isinstance(other, coordinate):
            other = coordinate(other)
        return coordinate(self.value / other.value)

    def __floordiv__(self, other):
        if not isinstance(other, coordinate):
            other = coordinate(other)
        return coordinate(self.value // other.value)  

    def __pow__(self, other):
        if not isinstance(other, coordinate):
            other = coordinate(other)
            
        return coordinate(self.value ** other.value)

    def __mod__(self, other):
        if not isinstance(other, coordinate):
            other = coordinate(other)
        return coordinate(self.value % other.value)   
        
    def __radd__(self, other):
        other = coordinate(other)
        return coordinate(other.value + self.value)   

    def __rsub__(self, other):
        other = coordinate(other)
        return coordinate(other.value - self.value)

    def __rmul__(self, other):
        other = coordinate(other)
        return coordinate(other.value * self.value)

    def __rtruediv__(self, other):
        other = coordinate(other)
        return coordinate(other.value / self.value)

    def __rfloordiv__(self, other):
        other = coordinate(other)
        return coordinate(other.value // self.value)  

    def __rpow__(self, other):
        # raise ValueError('POWER: this is not coordinate')
        other = coordinate(other)
        return coordinate(other.value ** self.value) 

    def __rmod__(self, other):
        other = coordinate(other)
        return coordinate(other.value % self.value)   

    def __neg__ (self):
        return coordinate(-self.value)   

    def __pos__ (self):
        return  coordinate(self.value)

    def __abs__(self):
        return coordinate(abs(self.value))

    def invert (self):
        return coordinate(1 / self.value)   
        
    def convertVal(self, other):
        thisVal = round(self.value, coordinate.p)
        try:
            otherVal = round(other.value, coordinate.p)
            return thisVal, otherVal
        except:
            if isinstance(other, int):
                otherVal = other
            elif isinstance(other, float):
                otherVal = round(other, coordinate.p)
            elif isinstance(other, str):
                otherVal = round(float(other), coordinate.p)
            else:
                raise ValueError('Unidentifiable number type: '+str(other.__class__))
            return thisVal, otherVal

        # if isinstance(other, coordinate):
        #     otherVal = round(other.value, coordinate.p)
        # elif isinstance(other, int):
        #     otherVal = other
        # elif isinstance(other, float):
        #     otherVal = round(other, coordinate.p)
        # elif isinstance(other, str):
        #     otherVal = round(float(other), coordinate.p)
        # else:
        #     raise ValueError('Unidentifiable number type: '+str(other.__class__))
        # return thisVal, otherVal

    def __eq__(self, other):
        # thisVal, otherVal = self.convertVal(other)
        # return thisVal == otherVal
        try:
            return coordinate.isclose(self.value, other.value)
        except AttributeError:
            pass
        try:
            return coordinate.isclose(self.value, other)
        except:
            pass
        try:
            return coordinate.isclose(self.value, float(other))
        except:
            pass
        raise ValueError('Unidentifiable number type: '+str(other.__class__))

        
    def __ne__(self, other):
        return not self == other

    def __lt__(self,other):
        # thisVal, otherVal = self.convertVal(other)
        # return thisVal < otherVal
        try:
            return self.value - other.value < coordinate.negtol * (10 ** ceil(log10(max(abs(self.value), abs(other.value)) + 1)))
        except:
            pass
        try:
            return self.value - other < coordinate.negtol * (10 ** ceil(log10(max(abs(self.value), abs(other)) + 1)))
        except:
            pass
        try:
            otherVal = float(other)
            return self.value - otherVal < coordinate.negtol * (10 ** ceil(log10(max(abs(self.value), abs(otherVal)) + 1)))
        except:
            pass
        raise ValueError('Unidentifiable number type: '+str(other.__class__))

    def __gt__(self,other):
        # thisVal, otherVal = self.convertVal(other)
        # return thisVal > otherVal
        try:
            return self.value - other.value > coordinate.tol * (10 ** ceil(log10(max(abs(self.value), abs(other.value)) + 1)))
        except:
            pass
        try:
            return self.value - other > coordinate.tol * (10 ** ceil(log10(max(abs(self.value), abs(other)) + 1)))
        except:
            pass
        try:
            otherVal = float(other)
            return self.value - otherVal > coordinate.tol * (10 ** ceil(log10(max(abs(self.value), abs(otherVal)) + 1)))
        except:
            pass
        raise ValueError('Unidentifiable number type: '+str(other.__class__))

    def __le__(self,other):
        # thisVal, otherVal = self.convertVal(other)
        # return thisVal <= otherVal
        try:
            return self.value - other.value <= coordinate.tol * (10 ** ceil(log10(max(abs(self.value), abs(other.value)) + 1)))
        except:
            pass
        try:
            return self.value - other <= coordinate.tol * (10 ** ceil(log10(max(abs(self.value), abs(other)) + 1)))
        except:
            pass
        try:
            otherVal = float(other)
            return self.value - otherVal <= coordinate.tol * (10 ** ceil(log10(max(abs(self.value), abs(otherVal)) + 1)))
        except:
            pass
        raise ValueError('Unidentifiable number type: '+str(other.__class__))

    def __ge__(self,other):
        # thisVal, otherVal = self.convertVal(other)
        # return thisVal >= otherVal
        try:
            return self.value - other.value >= coordinate.negtol * (10 ** ceil(log10(max(abs(self.value), abs(other.value)) + 1)))
        except:
            pass
        try:
            return self.value - other >= coordinate.negtol * (10 ** ceil(log10(max(abs(self.value), abs(other)) + 1)))
        except:
            pass
        try:
            otherVal = float(other)
            return self.value - otherVal >= coordinate.negtol * (10 ** ceil(log10(max(abs(self.value), abs(otherVal)) + 1)))
        except:
            pass
        raise ValueError('Unidentifiable number type: '+str(other.__class__))

    def __rshift__(self, other):
        if isinstance(other, coordinate):
            return coordinate(int(self.value) >> int(other.value))
        return coordinate(int(self.value) >> int(other))
        
    def compareValue(self, other):
        thisVal, otherVal = self.convertVal(other)
        if thisVal == otherVal:
            return 0
        elif thisVal > otherVal:
            return 1
        return -1




c0 = coordinate(0)
c1 = coordinate(1)
c2 = coordinate(2)
cn1 = coordinate(-1)
c0p5 = coordinate('0.5')

def isint(num):
    """
    External function for checking whether a coordinate is integer
    """
    if isinstance(num, coordinate) or isinstance(num, float):
        if int(num) == num:
            return True
    elif isinstance(num, int):
        return True
    return False