# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import range
from builtins import super
from builtins import str
from builtins import object


class key(object):
    """
    Key class.

    Used for marking individuals within a rule's lhs or rhs forms for description references 
    or rule predicates and directives.

    The key object and the individual should have a circular relationship. The key is stored
    as `ind_obj.key` and the individual is stored as `key_obj.ind`.

    A key object name must be unique within a rule.

    **Object Parameters:** (not including inherited attributes)

    * **name** (str) -- the key name.
    * **ind** (:class:`~sortal.ind.individual.individual`) -- the individual associated with the key.
    * **properties** (list of str) -- the special properties for some specific purposes.
        e.g. ">" and "<" properties for bound predicate.
    """
    type = 'struct-key'
    
    def __init__(self, keyName):
        self.name = keyName
        self.ind = None
        self.properties = []
        
        
    def __repr__(self):
        return '#' + self.name

        
    def __str__(self):
        return '#' + self.name
        
    def getBoundString(self):
        """
        Get the string representation of the key specifically for bound predicate
        """
        prefix = '>' if 0 in self.properties else ''
        postfix = '<' if 1 in self.properties else '' 
        return prefix + '#' + self.name + postfix
        
        
    def duplicate(self):
        """
        create a duplicate
        """
        dupKey = key(self.name)
        dupKey.ind = self.ind
        return dupKey
        
    @staticmethod
    def createKey(name, i):
        """
        create a new key object with the associated individual i
        """
        k = key(name)
        k.ind = i
        i.key = k
        return k
                
    def setIndividual(self, i):
        """
        set the individual of this key to be the individual i
        """
        self.ind = i
        i.key = self
        
    def addProperty(self, prop):
        """
        add a property to the key e.g. "<" or ">" for bound predicate
        """
        if isinstance(prop, list):
            self.properties += prop
        else:
            self.properties.append(prop)

