# -*- coding: utf-8 -*-

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
        
from builtins import str
from builtins import object

from .coordinate import cn1
class splayTree(object):
    """
    splay tree for plane segment classification
    """
    class node(object):
        """
        a node for splay tree
        """
        def __init__(self, content, parent = None, pred = None, succ = None):
            self.type = 'struct-splayTree.node'
            if 'ind' in content.type:
                raise ValueError('content must not be an individual')
            self.content = content
            self.parent = parent
            self.left = None
            self.right = None
            self.pred = pred
            self.succ = succ
            
        def __eq__(self, other):
            if other is None or other.__class__ != self.__class__:
                return False
            if self.content == other.content:
                return True
            return False
        
        def __str__(self):
            return 'node-'+str(self.content)
            
        def printSucc(self, printSelf = True):
            if printSelf:
                print('---',self.content)
            if self.succ is not None:
                self.succ.printSucc()
    
        def printPred(self, printSelf = True):
            if self.pred is not None:
                self.pred.printPred()
            if printSelf:
                print('---',self.content)
                
    def __init__(self):
        self.type = 'struct-splayTree'
        self.root = None
        self.size = 0
    
    def __len__(self):
        return self.size
        
    def empty(self):
        return self.size == 0
        
    def printTree(self, node = 0, direction = '\t'):
        if node is 0:
            self.printTree(self.root)
        elif node is not None:
            print(str(node.content), direction)
            self.printTree(node.left, direction+' left')
            self.printTree(node.right, direction+' right')
            
    def printPredSucc(self):
        self.root.printPred(False)
        self.root.printSucc(True)
                
    def purge(self):
        self.root = None
        self.size = 0
    
    @staticmethod
    def rotateLeft(y):
        x = y.right
        z = y.parent

        if z is not None:
            if y == z.left:
                z.left = x
            else: 
                z.right = x
                
        y.right = x.left
        x.left = y
        x.parent = z
        y.parent = x
        if y.right is not None:
            y.right.parent = y
            
    @staticmethod
    def rotateRight(y):
        x = y.left
        z = y.parent

        if z is not None:
            if y == z.left:
                z.left = x
            else: 
                z.right = x
                
        y.left = x.right
        x.right = y
        x.parent = z
        y.parent = x
        if y.left is not None:
            y.left.parent = y
            
    def splay(self, x):
        while x.parent is not None:
            if x == x.parent.left:
                if x.parent.parent is None:
                    #terminating single rotation "zig" 
                    self.rotateRight(x.parent)
                elif x.parent == x.parent.parent.left:
                    #two single rotations "zig-zig"
                    self.rotateRight(x.parent.parent)
                    self.rotateRight(x.parent)
                else:
                    #double rotation "zig-zag"
                    self.rotateRight(x.parent)
                    self.rotateLeft(x.parent)
            
            else:
                #symmetric cases
                if x.parent.parent is None:
                    self.rotateLeft(x.parent)
                elif x.parent == x.parent.parent.right:
                    self.rotateLeft(x.parent.parent)
                    self.rotateLeft(x.parent)
                else:
                    self.rotateLeft(x.parent)
                    self.rotateRight(x.parent)

    def insert(self, obj):
        if self.empty():
            self.root = splayTree.node(obj)
            self.size = 1
            return
        node = self.root
        while True:
            side = node.content.splayCompare(obj) * cn1
            if side < 0:
                if node.left is not None:
                    node = node.left
                else:
                    self.root = splayTree.node(obj, node, node.pred, node)
                    node.left = self.root
                    if node.pred is not None: 
                        node.pred.succ = node.left
                    node.pred = node.left
                    break
            elif node.right is not None:
                node = node.right
            else:
                self.root = splayTree.node(obj, node, node, node.succ)
                node.right = self.root
                if node.succ is not None:
                    node.succ.pred = node.right
                node.succ = node.right
                break
        self.size+=1
        self.splay(self.root)

    def find(self, obj):
        if self.empty():
            return None
        node = self.root
        while True:
            side = node.content.splayCompare(obj) * cn1
            if side < 0 and node.left is not None:
                node = node.left
            elif side > 0 and node.right is not None:
                node = node.right
            elif side == 0 and node.pred is not None and node.pred.content.splayCompare(obj) == 0:
                node = node.pred
            else:
                self.root = node
                break
        self.splay(self.root)
        return self.root
        
    def findMin(self):
        if self.empty():
            return None
        node = self.root
        while True:
            if node.left is not None:
                node = node.left
            else:
                break
        return node        
        
    def extract(self, node):
        self.splay(node)
        # disconnect node and left subtree
        lt = node.pred #right most node in left tree 
        if lt is not None:
            node.left.parent = None
            self.splay(lt)
            lt.succ = None
        
        # disconnect node and right subtree 
        rt = node.succ #left most node in right tree
        if rt is not None:
            node.right.parent = None
            # make lt the left child of rt 
            rt.pred = lt
            rt.left = lt
            if lt is not None: 
                lt.succ = rt
                lt.parent = rt
            self.root = node.right
        else:
            self.root = lt
        self.size -= 1
        return node.content
        
    def extractMin(self):
        if self.empty():
            return None
        node = self.root
        while True:
            if node.left is not None:
                node = node.left
            else:
                break
        return self.extract(node)       

    def delete(self, object):
        if self.empty():
            return None
        node = self.root
        while True:
            if node.content == object:
                break
            side = node.content.splayCompare(object) * -1
            if side < 0 and node.left is not None:
                node = node.left
            elif side > 0 and node.right is not None:
                node = node.right
            else: 
                return False
        
        self.extract(node)
        return True

    def initSentinelTree(self, start, end):
        self.root = splayTree.node(start, None, None, None)
        self.root.right = splayTree.node(end, self.root, self.root, None)
        self.root.succ = self.root.right
        self.size = 2

    def swap(self, node):
        temp = node.left
        node.left = node.right
        node.right = temp
        if node.left is not None: 
            self.swap(node.left)
        if node.right is not None:
            self.swap(node.right)
        
    def reverse(self, x, y):
        self.root = y
        self.splay(self.root)
        if x == y:
            return
        #disconnect x and predecessor
        left = x.pred #right most node in left tree
        if left is not None:
            left.succ = None
            x.left.parent = None
            self.splay(left)
            x.pred = x.left = None
        
        # disconnect y and successor
        right = y.succ # left most node in right tree
        if (right != None):
            right.pred = None
            y.right.parent = None
            self.splay(right)
            y.succ = y.right = None
        
        # reverse pred and succ links between x and y
        node  = x
        while node is not None:
            temp = node.pred
            node.pred = node.succ
            node.succ = temp
            node = node.pred
        
        # reverse left and right links between x and y recursively,
        # starting at y, which is the current root
        self.swap(y)
        # make left the left child of y
        y.pred = y.left = left
        if left is not None: 
            left.parent = y
            left.succ = left.parent
        x.succ = x.right = right
        if right is not None:
            right.parent = x
            right.pred = right.parent