# -*- coding: utf-8 -*-

from __future__ import division
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import

from copy import deepcopy

class paramSet():
    """
    paramSet: transformation for description class.

    The parameters matches are saved in self.parameters in dictionary format 
    with the key being a string in the format of "sort_name|parameter_name" and 
    value being the matched value of the parameters.

    All the unresolved conditions are saved in self.conditions in dictionary format. 
    If a condition is satisfied, it should be removed from the list.

    **Object Parameters:**

    * **parameters** (dict) -- the parameters dictionary.
    * **conditions** (dict) -- the conditions dictionary.
    * **results** (dict) -- results of previously evaluated description. For having a consistent rand() function result transformation.
    * **keys** (dict) -- dictionary of object keys/tags in the rule.
    * **lhs** (form) -- the transfomred lhs of the rule.
    * **rhs** (form) -- the transfomred rhs of the rule.
    * **id** (str) -- the disjunctive sort of this paramSet.
    * **current** (str) -- the current sort that is being assessed.
    * **rule** (rule) -- the rule this paramSet is applicable to (unused?).
    * **transform** (rule) -- the geometric transformation accompanying this paramSet (unused?).


    """
    type = 'paramSet'
    def __init__(self):
        self.parameters = {}
        self.conditions= {}
        self.results = {}
        self.keys = {}
        self.lhs = None
        self.rhs = None
        
        self.rule = None
        self.transform = None
        
        self.id = None
        self.current = None
    
    def isEmpty(self):
        return len(self.parameters) == 0
        
    def __eq__(self, other):
        return self.parameters == other.parameters and self.conditions == other.conditions
        
    def __str__(self):
        return str(self.parameters)
        return str(self.parameters)+'|'+ str(self.conditions)

    def __repr__(self):
        return str(self.parameters)
        
    def duplicate(self):
        """
        create a duplicate transformation
        """
        newTr = paramSet()
        newTr.parameters = self.parameters
        newTr.conditions = self.conditions
        newTr.lhs = self.lhs
        newTr.rhs = self.rhs
        return newTr
            
        
    def printVal(self):
        """
        print all the parameters matches
        """
        for i in self.parameters:
            print(i, self.parameters[i])
        
    def add(self, sort, name, value):
        """
        add a parameter into the paramSet
        """
        if name == 'value':
            raise ValueError
        key = sort.name+'|'+name
        if key not in self.parameters:

            self.parameters[key] = value

        elif not self.parameters[key] == value:
            raise ParameterNameError('parameter name '+name+' is repeated in the left hand side')
        return True
        
    def merge(self, other):
        """
        merge two paramSets (merging parameters and conditions)
        """
        for i in other.parameters:
            if not i in self.parameters:
                self.parameters[i] = other.parameters[i]
        for i in other.conditions:
            if not i in self.parameters:
                self.conditions[i] = other.conditions[i]
        
    def retrieve(self, sort, name):
        """
        retrieve a parameter match
        """
        key = sort.name+'|'+name
        if key not in self.parameters and self.lhs is not None:
            return None
        result = self.parameters[key]
        return result
        
    def getState(self, sort):
        """
        get a condition
        """
        return self.conditions[str(delayable)]
        
    def delay(self, delayable, sort, value):
        """
        register a condition delay
        """
        self.conditions[str(delayable) + '|' + str(value)] = [delayable, sort, value]
    
    def resolveDelays(self):
        """
        resolve all possible condition delays
        """
        result = True
        delayables = list(self.conditions.keys())
        for delay in delayables:
            if not self.conditions[delay][0].delayedMatch(self, self.conditions[delay]):
                result = False
        return result
    
    def resolveSimpleDelays(self):
        """
        resolve all possible simple condition delays: those condtions that do not require
        informations from the geometry
        """
        result = True
        delayables = list(self.conditions.keys())
        for delay in delayables:
            check = self.conditions[delay][0].simpleDelayedMatch(self, self.conditions[delay])
            if not check:
                result = False
            elif check is True:
                self.conditions.pop(delay)
                
        return result
    
    def apply(self, element):
        """
        apply the description transformation on a description
        """
        return element.edit(self)

class ParameterNameError(Exception):
    def __init__(self, message = ''):
        super(ParameterNameError, self).__init__(message)
