# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import


from builtins import object
class transformSet(object):
    """
    transformSet class: compilation of the various separate transformations of different mappings
    on a single shape for one rule

    **Object Parameters:**

    * **map** (dict) -- dictionary of transformations with the keys being the transformations' corresponding mappings.
    * **excess** (dict) -- <<unused>>.
    * **current** (str) -- the current sort that is being assessed.

    """
    repository = {}
    type = 'transformSet'
    def __init__(self):
        self.map = {}
        self.excess = {}
        self.current = None
        return
    
    def printTransf(self):
        """
        print the transformation for debugging
        """
        print('transforms:')
        for i in self.map:
            print('--', i)
            self.map[i].printTransf()
            
    def duplicate(self):
        """
        create a duplicate transformation
        """
        newTr = transformSet()
        for i in self.map:
            newTr.map[i] = self.map[i].duplicate()
        return newTr
        
    def retrieve(self, var):
        return transformSet.repository[var]
    
    def deposit(self, var):
        transformSet.repository[var] = self
        
    def retrieveTransform(self, clss):
        """
        retrieve a transformation of a mapping
        """
        try:
            if self.current is None or clss.type == 'map-parametric':
                return self.map[clss.type]
            else:
                code = self.current+'=|='+clss.type
                if code in self.map:
                    return self.map[code]
                elif clss.type in self.map:
                    return self.map[clss.type]
                else:
                    return None
        except KeyError:
            return None
        
    def add(self, transform):
        """
        add a transformation into the transformSet
        """
        if transform.id is None:
            self.map[transform.type] = transform
        else:
            self.map[transform.id+'=|='+transform.type] = transform
        
    def addExcess(self, transform):
        self.excess[transform.__class__] = transform
        